# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.classify_config import ClassifyConfigParams
from ..types.classifier import Classifier
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_by import SortBy
from ..types.sort_dir import SortDir
from .raw_client import AsyncRawClassifiersClient, RawClassifiersClient
from .types.classifiers_list_response import ClassifiersListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ClassifiersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawClassifiersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawClassifiersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawClassifiersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClassifiersListResponse:
        """
        List all classifiers.

        Returns a summary of each classifier. Use `GET /classifiers/{id}` to retrieve the full object including `draftVersion`.

        Parameters
        ----------
        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClassifiersListResponse
            Successfully retrieved classifiers

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.classifiers.list(
            next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
        )
        """
        _response = self._raw_client.list(
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            sort_by=sort_by,
            sort_dir=sort_dir,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        name: str,
        clone_classifier_id: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Classifier:
        """
        Create a new classifier.

        Parameters
        ----------
        name : str
            The name of the classifier.

        clone_classifier_id : typing.Optional[str]
            The ID of an existing classifier to clone. If provided, the new classifier will be created with the same config as the classifier with this ID. Cannot be provided together with `config`.

            Example: `"cl_BMdfq_yWM3sT-ZzvCnA3f"`

        config : typing.Optional[ClassifyConfigParams]
            The configuration for the classifier. Cannot be provided together with `cloneClassifierId`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Classifier
            Classifier created successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.classifiers.create(
            name="Document Classifier",
            config={
                "classifications": [
                    {
                        "id": "invoice",
                        "type": "invoice",
                        "description": "An invoice or bill for goods or services",
                    },
                    {
                        "id": "receipt",
                        "type": "receipt",
                        "description": "A receipt confirming payment",
                    },
                    {
                        "id": "other",
                        "type": "other",
                        "description": "Any other document type",
                    },
                ]
            },
        )
        """
        _response = self._raw_client.create(
            name=name, clone_classifier_id=clone_classifier_id, config=config, request_options=request_options
        )
        return _response.data

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Classifier:
        """
        Get details of a classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to get.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Classifier
            Classifier details

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.classifiers.retrieve(
            id="classifier_id_here",
        )
        """
        _response = self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Classifier:
        """
        Update an existing classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to update.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        name : typing.Optional[str]
            The new name of the classifier.

        config : typing.Optional[ClassifyConfigParams]
            The new configuration for the classifier. This will update the draft version of the classifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Classifier
            Classifier updated successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.classifiers.update(
            id="classifier_id_here",
            name="Document Classifier v2",
        )
        """
        _response = self._raw_client.update(id, name=name, config=config, request_options=request_options)
        return _response.data


class AsyncClassifiersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawClassifiersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawClassifiersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawClassifiersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClassifiersListResponse:
        """
        List all classifiers.

        Returns a summary of each classifier. Use `GET /classifiers/{id}` to retrieve the full object including `draftVersion`.

        Parameters
        ----------
        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClassifiersListResponse
            Successfully retrieved classifiers

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.classifiers.list(
                next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            sort_by=sort_by,
            sort_dir=sort_dir,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        name: str,
        clone_classifier_id: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Classifier:
        """
        Create a new classifier.

        Parameters
        ----------
        name : str
            The name of the classifier.

        clone_classifier_id : typing.Optional[str]
            The ID of an existing classifier to clone. If provided, the new classifier will be created with the same config as the classifier with this ID. Cannot be provided together with `config`.

            Example: `"cl_BMdfq_yWM3sT-ZzvCnA3f"`

        config : typing.Optional[ClassifyConfigParams]
            The configuration for the classifier. Cannot be provided together with `cloneClassifierId`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Classifier
            Classifier created successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.classifiers.create(
                name="Document Classifier",
                config={
                    "classifications": [
                        {
                            "id": "invoice",
                            "type": "invoice",
                            "description": "An invoice or bill for goods or services",
                        },
                        {
                            "id": "receipt",
                            "type": "receipt",
                            "description": "A receipt confirming payment",
                        },
                        {
                            "id": "other",
                            "type": "other",
                            "description": "Any other document type",
                        },
                    ]
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name, clone_classifier_id=clone_classifier_id, config=config, request_options=request_options
        )
        return _response.data

    async def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Classifier:
        """
        Get details of a classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to get.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Classifier
            Classifier details

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.classifiers.retrieve(
                id="classifier_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Classifier:
        """
        Update an existing classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to update.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        name : typing.Optional[str]
            The new name of the classifier.

        config : typing.Optional[ClassifyConfigParams]
            The new configuration for the classifier. This will update the draft version of the classifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Classifier
            Classifier updated successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.classifiers.update(
                id="classifier_id_here",
                name="Document Classifier v2",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, name=name, config=config, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class SplitAdvancedOptionsSplitMethod(enum.StrEnum):
    """
    The method to use for splitting documents. `high_precision` is more accurate but slower, while `basic_precision` is faster but less precise.

    **Deprecated:** For `splitting_light` >= 1.3.0 and `splitting_performance` >= 1.5.0, this field has no impact and is ignored if provided. It is still accepted for compatibility with older integrations.
    """

    HIGH_PRECISION = "high_precision"
    BASIC_PRECISION = "basic_precision"
    _UNKNOWN = "__SPLITADVANCEDOPTIONSSPLITMETHOD_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "SplitAdvancedOptionsSplitMethod":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        high_precision: typing.Callable[[], T_Result],
        basic_precision: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is SplitAdvancedOptionsSplitMethod.HIGH_PRECISION:
            return high_precision()
        if self is SplitAdvancedOptionsSplitMethod.BASIC_PRECISION:
            return basic_precision()
        return _unknown_member(self._value_)



"""
Extend SDK wrapper module with polling utilities and webhook verification.

This module provides extended versions of the generated SDK clients with:
- `create_and_poll()` methods for convenient polling
- Webhook signature verification utilities
- Custom error classes

Example:
    from extend_ai import Extend

    client = Extend(token="your-api-key")

    # Use create_and_poll for convenient polling
    result = client.extract_runs.create_and_poll(
        file={"url": "https://example.com/doc.pdf"},
        extractor={"id": "extractor_123"},
    )

    # Verify webhooks
    event = client.webhooks.verify_and_parse(body, headers, secret)
"""

from .client import AsyncExtend, Extend
from .errors import (
    PollingTimeoutError,
    SignedUrlNotAllowedError,
    WebhookParseError,
    WebhookPayloadFetchError,
    WebhookSignatureVerificationError,
)
from .polling import PollingOptions, calculate_backoff_delay, poll_until_done, poll_until_done_async
from .webhooks import RawWebhookEvent, SignedDataUrlPayload, WebhookEventWithSignedUrl, Webhooks

__all__ = [
    # Client
    "Extend",
    "AsyncExtend",
    # Webhooks
    "Webhooks",
    "RawWebhookEvent",
    "WebhookEventWithSignedUrl",
    "SignedDataUrlPayload",
    # Polling
    "PollingOptions",
    "poll_until_done",
    "poll_until_done_async",
    "calculate_backoff_delay",
    # Errors
    "PollingTimeoutError",
    "WebhookSignatureVerificationError",
    "WebhookParseError",
    "WebhookPayloadFetchError",
    "SignedUrlNotAllowedError",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.edit_config import EditConfigParams
from ..types.edit_run import EditRun
from .raw_client import AsyncRawEditRunsClient, RawEditRunsClient
from .requests.edit_runs_create_request_file import EditRunsCreateRequestFileParams
from .types.edit_runs_delete_response import EditRunsDeleteResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EditRunsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEditRunsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEditRunsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEditRunsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        file: EditRunsCreateRequestFileParams,
        config: typing.Optional[EditConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditRun:
        """
        Edit and manipulate PDF documents by detecting and filling form fields.

        The Edit Runs endpoint allows you to convert and edit documents and get an edit run ID that can be used to check status and retrieve results with the Get Edit Run endpoint.

        The request returns immediately with a `PROCESSING` status. Use webhooks or poll the Get Edit Run endpoint for results.

        Parameters
        ----------
        file : EditRunsCreateRequestFileParams
            The file to be edited. Files can be provided as a URL or an Extend file ID.

        config : typing.Optional[EditConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditRun
            Successfully initiated edit run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.edit_runs.create(
            file={"url": "https://example.com/form.pdf"},
            config={
                "instructions": "Fill out the form with the provided data",
                "advanced_options": {"flatten_pdf": True},
            },
        )
        """
        _response = self._raw_client.create(file=file, config=config, request_options=request_options)
        return _response.data

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EditRun:
        """
        Retrieve the status and results of an edit run.

        Use this endpoint to get results for an edit run that has already completed, or to check on the status of an edit run initiated via the [Create Edit Run](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/edit/create-edit-run) endpoint.

        Parameters
        ----------
        id : str
            The unique identifier for the edit run.

            Example: `"edr_xK9mLPqRtN3vS8wF5hB2cQ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditRun
            Successfully retrieved edit run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.edit_runs.retrieve(
            id="edit_run_id_here",
        )
        """
        _response = self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EditRunsDeleteResponse:
        """
        Delete an edit run and all associated data from Extend. This operation is permanent and cannot be undone.

        This endpoint can be used if you'd like to manage data retention on your own rather than relying on automated data retention policies, or to make one-off deletions for your downstream customers.

        Parameters
        ----------
        id : str
            The ID of the edit run to delete.

            Example: `"edr_xK9mLPqRtN3vS8wF5hB2cQ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditRunsDeleteResponse
            Successfully deleted edit run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.edit_runs.delete(
            id="edit_run_id_here",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data


class AsyncEditRunsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEditRunsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEditRunsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEditRunsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        file: EditRunsCreateRequestFileParams,
        config: typing.Optional[EditConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditRun:
        """
        Edit and manipulate PDF documents by detecting and filling form fields.

        The Edit Runs endpoint allows you to convert and edit documents and get an edit run ID that can be used to check status and retrieve results with the Get Edit Run endpoint.

        The request returns immediately with a `PROCESSING` status. Use webhooks or poll the Get Edit Run endpoint for results.

        Parameters
        ----------
        file : EditRunsCreateRequestFileParams
            The file to be edited. Files can be provided as a URL or an Extend file ID.

        config : typing.Optional[EditConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditRun
            Successfully initiated edit run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.edit_runs.create(
                file={"url": "https://example.com/form.pdf"},
                config={
                    "instructions": "Fill out the form with the provided data",
                    "advanced_options": {"flatten_pdf": True},
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(file=file, config=config, request_options=request_options)
        return _response.data

    async def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EditRun:
        """
        Retrieve the status and results of an edit run.

        Use this endpoint to get results for an edit run that has already completed, or to check on the status of an edit run initiated via the [Create Edit Run](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/edit/create-edit-run) endpoint.

        Parameters
        ----------
        id : str
            The unique identifier for the edit run.

            Example: `"edr_xK9mLPqRtN3vS8wF5hB2cQ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditRun
            Successfully retrieved edit run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.edit_runs.retrieve(
                id="edit_run_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EditRunsDeleteResponse:
        """
        Delete an edit run and all associated data from Extend. This operation is permanent and cannot be undone.

        This endpoint can be used if you'd like to manage data retention on your own rather than relying on automated data retention policies, or to make one-off deletions for your downstream customers.

        Parameters
        ----------
        id : str
            The ID of the edit run to delete.

            Example: `"edr_xK9mLPqRtN3vS8wF5hB2cQ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditRunsDeleteResponse
            Successfully deleted edit run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.edit_runs.delete(
                id="edit_run_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .legacy_document_processor_run_credits import LegacyDocumentProcessorRunCredits
from .legacy_processor_run_summary_type import LegacyProcessorRunSummaryType
from .legacy_processor_status import LegacyProcessorStatus


class LegacyProcessorRunSummary(UncheckedBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the processor run.
    
    Example: `"exr_Zk9mNP12Qw4-yTv8BdR3H"`
    """

    status: LegacyProcessorStatus
    processor_id: typing_extensions.Annotated[str, FieldMetadata(alias="processorId")] = pydantic.Field(
        alias="processorId"
    )
    """
    The ID of the processor that was run. Will always start with "ex_".
    
    Example: `"ex_BMdfq_yWM3sT-ZzvCnA3f"`
    """

    processor_name: typing_extensions.Annotated[str, FieldMetadata(alias="processorName")] = pydantic.Field(
        alias="processorName"
    )
    """
    The name of the processor that was run.
    
    Example: `"Invoice Extractor"`
    """

    processor_version_id: typing_extensions.Annotated[str, FieldMetadata(alias="processorVersionId")] = pydantic.Field(
        alias="processorVersionId"
    )
    """
    The ID of the processor version that was run. Will always start with "exv_".
    
    Example: `"exv_Zk9mNP12Qw4-yTv8BdR3H"`
    """

    type: typing.Optional[LegacyProcessorRunSummaryType] = pydantic.Field(default=None)
    """
    The type of processor that was run:
    * `"EXTRACT"` - Extraction processor
    * `"SPLITTER"` - Splitting processor
    * `"CLASSIFY"` - Classification processor
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )
    """
    The time (in UTC) at which the processor run was created. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T15:29:55Z"`
    """

    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")] = pydantic.Field(
        alias="updatedAt"
    )
    """
    The time (in UTC) at which the processor run was last updated. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T16:45:00Z"`
    """

    usage: typing.Optional[LegacyDocumentProcessorRunCredits] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_dir import SortDir
from ..types.webhook_subscription import WebhookSubscription
from ..types.webhook_subscription_event_type import WebhookSubscriptionEventType
from ..types.webhook_subscription_resource_type import WebhookSubscriptionResourceType
from .raw_client import AsyncRawWebhookSubscriptionsClient, RawWebhookSubscriptionsClient
from .types.webhook_subscriptions_delete_response import WebhookSubscriptionsDeleteResponse
from .types.webhook_subscriptions_list_response import WebhookSubscriptionsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WebhookSubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWebhookSubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWebhookSubscriptionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWebhookSubscriptionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        webhook_endpoint_id: typing.Optional[str] = None,
        resource_id: typing.Optional[str] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSubscriptionsListResponse:
        """
        List webhook subscriptions. You can filter by `webhookEndpointId` to see all subscriptions for a given endpoint, or by `resourceId` to see all subscriptions for a given resource.

        Parameters
        ----------
        webhook_endpoint_id : typing.Optional[str]
            Filter subscriptions by the webhook endpoint they belong to.

        resource_id : typing.Optional[str]
            Filter subscriptions by the resource they are scoped to.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscriptionsListResponse
            List of webhook subscriptions

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_subscriptions.list(
            next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
        )
        """
        _response = self._raw_client.list(
            webhook_endpoint_id=webhook_endpoint_id,
            resource_id=resource_id,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        webhook_endpoint_id: str,
        resource_type: WebhookSubscriptionResourceType,
        resource_id: str,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSubscription:
        """
        Create a resource-scoped webhook subscription on an existing webhook endpoint.

        Subscriptions let you receive events for a specific resource (e.g., a single extractor or workflow) rather than all resources of that type. The `enabledEvents` must be valid for the given `resourceType` and the endpoint's `apiVersion`.

        If a subscription already exists for the same endpoint and resource, it will be updated with the new `enabledEvents` instead of creating a duplicate.

        Parameters
        ----------
        webhook_endpoint_id : str
            The ID of the webhook endpoint to attach this subscription to.

        resource_type : WebhookSubscriptionResourceType

        resource_id : str
            The ID of the resource to scope this subscription to.

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the given `resourceType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscription
            Successfully created webhook subscription

        Examples
        --------
        from extend_ai import (
            Extend,
            WebhookSubscriptionEventType,
            WebhookSubscriptionResourceType,
        )

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_subscriptions.create(
            webhook_endpoint_id="wh_Xj8mK2pL9nR4vT7qY5wZ",
            resource_type=WebhookSubscriptionResourceType.EXTRACTOR,
            resource_id="ex_Xj8mK2pL9nR4vT7qY5wZ",
            enabled_events=[
                WebhookSubscriptionEventType.EXTRACT_RUN_PROCESSED,
                WebhookSubscriptionEventType.EXTRACT_RUN_FAILED,
            ],
        )
        """
        _response = self._raw_client.create(
            webhook_endpoint_id=webhook_endpoint_id,
            resource_type=resource_type,
            resource_id=resource_id,
            enabled_events=enabled_events,
            request_options=request_options,
        )
        return _response.data

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> WebhookSubscription:
        """
        Retrieve a webhook subscription by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscription
            Retrieved webhook subscription

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_subscriptions.retrieve(
            id="webhook_subscription_id_here",
        )
        """
        _response = self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSubscription:
        """
        Update the enabled events on a webhook subscription.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to update.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the subscription's resource type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscription
            Successfully updated webhook subscription

        Examples
        --------
        from extend_ai import Extend, WebhookSubscriptionEventType

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_subscriptions.update(
            id="webhook_subscription_id_here",
            enabled_events=[WebhookSubscriptionEventType.EXTRACT_RUN_PROCESSED],
        )
        """
        _response = self._raw_client.update(id, enabled_events=enabled_events, request_options=request_options)
        return _response.data

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WebhookSubscriptionsDeleteResponse:
        """
        Delete a webhook subscription. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to delete.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscriptionsDeleteResponse
            Successfully deleted webhook subscription

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_subscriptions.delete(
            id="webhook_subscription_id_here",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data


class AsyncWebhookSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWebhookSubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWebhookSubscriptionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWebhookSubscriptionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        webhook_endpoint_id: typing.Optional[str] = None,
        resource_id: typing.Optional[str] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSubscriptionsListResponse:
        """
        List webhook subscriptions. You can filter by `webhookEndpointId` to see all subscriptions for a given endpoint, or by `resourceId` to see all subscriptions for a given resource.

        Parameters
        ----------
        webhook_endpoint_id : typing.Optional[str]
            Filter subscriptions by the webhook endpoint they belong to.

        resource_id : typing.Optional[str]
            Filter subscriptions by the resource they are scoped to.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscriptionsListResponse
            List of webhook subscriptions

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_subscriptions.list(
                next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            webhook_endpoint_id=webhook_endpoint_id,
            resource_id=resource_id,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        webhook_endpoint_id: str,
        resource_type: WebhookSubscriptionResourceType,
        resource_id: str,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSubscription:
        """
        Create a resource-scoped webhook subscription on an existing webhook endpoint.

        Subscriptions let you receive events for a specific resource (e.g., a single extractor or workflow) rather than all resources of that type. The `enabledEvents` must be valid for the given `resourceType` and the endpoint's `apiVersion`.

        If a subscription already exists for the same endpoint and resource, it will be updated with the new `enabledEvents` instead of creating a duplicate.

        Parameters
        ----------
        webhook_endpoint_id : str
            The ID of the webhook endpoint to attach this subscription to.

        resource_type : WebhookSubscriptionResourceType

        resource_id : str
            The ID of the resource to scope this subscription to.

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the given `resourceType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscription
            Successfully created webhook subscription

        Examples
        --------
        import asyncio

        from extend_ai import (
            AsyncExtend,
            WebhookSubscriptionEventType,
            WebhookSubscriptionResourceType,
        )

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_subscriptions.create(
                webhook_endpoint_id="wh_Xj8mK2pL9nR4vT7qY5wZ",
                resource_type=WebhookSubscriptionResourceType.EXTRACTOR,
                resource_id="ex_Xj8mK2pL9nR4vT7qY5wZ",
                enabled_events=[
                    WebhookSubscriptionEventType.EXTRACT_RUN_PROCESSED,
                    WebhookSubscriptionEventType.EXTRACT_RUN_FAILED,
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            webhook_endpoint_id=webhook_endpoint_id,
            resource_type=resource_type,
            resource_id=resource_id,
            enabled_events=enabled_events,
            request_options=request_options,
        )
        return _response.data

    async def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WebhookSubscription:
        """
        Retrieve a webhook subscription by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscription
            Retrieved webhook subscription

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_subscriptions.retrieve(
                id="webhook_subscription_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSubscription:
        """
        Update the enabled events on a webhook subscription.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to update.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the subscription's resource type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscription
            Successfully updated webhook subscription

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend, WebhookSubscriptionEventType

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_subscriptions.update(
                id="webhook_subscription_id_here",
                enabled_events=[WebhookSubscriptionEventType.EXTRACT_RUN_PROCESSED],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, enabled_events=enabled_events, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WebhookSubscriptionsDeleteResponse:
        """
        Delete a webhook subscription. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to delete.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSubscriptionsDeleteResponse
            Successfully deleted webhook subscription

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_subscriptions.delete(
                id="webhook_subscription_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .classify_step_run import ClassifyStepRun
from .conditional_extract_step_run import ConditionalExtractStepRun
from .external_data_validation_step_run import ExternalDataValidationStepRun
from .extract_step_run import ExtractStepRun
from .merge_extract_step_run import MergeExtractStepRun
from .parse_step_run import ParseStepRun
from .rule_validation_step_run import RuleValidationStepRun
from .split_step_run import SplitStepRun

StepRun = typing.Union[
    ParseStepRun,
    ExtractStepRun,
    ClassifyStepRun,
    SplitStepRun,
    MergeExtractStepRun,
    ConditionalExtractStepRun,
    RuleValidationStepRun,
    ExternalDataValidationStepRun,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from .created_at import CreatedAt


class EvaluationSetRunEntityVersion_ExtractorVersion(UncheckedBaseModel):
    """
    The version of the extractor, classifier, or splitter that was run.
    """

    object: typing.Literal["extractor_version"] = "extractor_version"
    id: str
    description: typing.Optional[str] = None
    version: str
    extractor_id: typing_extensions.Annotated[str, FieldMetadata(alias="extractorId")] = pydantic.Field(
        alias="extractorId"
    )
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EvaluationSetRunEntityVersion_ClassifierVersion(UncheckedBaseModel):
    """
    The version of the extractor, classifier, or splitter that was run.
    """

    object: typing.Literal["classifier_version"] = "classifier_version"
    id: str
    description: typing.Optional[str] = None
    version: str
    classifier_id: typing_extensions.Annotated[str, FieldMetadata(alias="classifierId")] = pydantic.Field(
        alias="classifierId"
    )
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EvaluationSetRunEntityVersion_SplitterVersion(UncheckedBaseModel):
    """
    The version of the extractor, classifier, or splitter that was run.
    """

    object: typing.Literal["splitter_version"] = "splitter_version"
    id: str
    description: typing.Optional[str] = None
    version: str
    splitter_id: typing_extensions.Annotated[str, FieldMetadata(alias="splitterId")] = pydantic.Field(
        alias="splitterId"
    )
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


EvaluationSetRunEntityVersion = typing_extensions.Annotated[
    typing.Union[
        EvaluationSetRunEntityVersion_ExtractorVersion,
        EvaluationSetRunEntityVersion_ClassifierVersion,
        EvaluationSetRunEntityVersion_SplitterVersion,
    ],
    UnionMetadata(discriminant="object"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from ...types.legacy_next_page_token import LegacyNextPageToken
from ...types.legacy_processor_run_summary import LegacyProcessorRunSummary


class ProcessorRunListResponse(UncheckedBaseModel):
    success: bool
    processor_runs: typing_extensions.Annotated[
        typing.List[LegacyProcessorRunSummary], FieldMetadata(alias="processorRuns")
    ] = pydantic.Field(alias="processorRuns")
    next_page_token: typing_extensions.Annotated[
        typing.Optional[LegacyNextPageToken], FieldMetadata(alias="nextPageToken")
    ] = pydantic.Field(alias="nextPageToken", default=None)

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.provided_processor_output import ProvidedProcessorOutputParams
from ..types.evaluation_set_item import EvaluationSetItem
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_by import SortBy
from ..types.sort_dir import SortDir
from .raw_client import AsyncRawEvaluationSetItemsClient, RawEvaluationSetItemsClient
from .requests.evaluation_set_items_create_request_items_item import EvaluationSetItemsCreateRequestItemsItemParams
from .types.evaluation_set_items_create_response import EvaluationSetItemsCreateResponse
from .types.evaluation_set_items_delete_response import EvaluationSetItemsDeleteResponse
from .types.evaluation_set_items_list_response import EvaluationSetItemsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EvaluationSetItemsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEvaluationSetItemsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEvaluationSetItemsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEvaluationSetItemsClient
        """
        return self._raw_client

    def list(
        self,
        evaluation_set_id: str,
        *,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EvaluationSetItemsListResponse:
        """
        List items in a specific evaluation set.

        Returns a summary of each evaluation set item. Use the [Get Evaluation Set Item](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/evaluation/get-evaluation-set-item) endpoint to get the full details of an evaluation set item.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItemsListResponse
            List of evaluation set items

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.evaluation_set_items.list(
            evaluation_set_id="evaluation_set_id_here",
            next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
        )
        """
        _response = self._raw_client.list(
            evaluation_set_id,
            sort_by=sort_by,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        evaluation_set_id: str,
        *,
        items: typing.Sequence[EvaluationSetItemsCreateRequestItemsItemParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EvaluationSetItemsCreateResponse:
        """
        Evaluation set items are the individual files and expected outputs that are used to evaluate the performance of a given extractor, classifier, or splitter in Extend. This endpoint will create new evaluation set items in Extend, which will be used during an evaluation run.

        **Limit:** You can create up to 100 items at a time.

        Learn more about how to create evaluation set items in the [Evaluation Sets](https://docs.extend.ai/2026-02-09/product/evaluation/overview) product page.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        items : typing.Sequence[EvaluationSetItemsCreateRequestItemsItemParams]
            An array of objects representing the evaluation set items to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItemsCreateResponse
            Evaluation set items created successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.evaluation_set_items.create(
            evaluation_set_id="evaluation_set_id_here",
            items=[
                {
                    "file_id": "file_xK9mLPqRtN3vS8wF5hB2cQ",
                    "expected_output": {
                        "value": {
                            "vendor_name": "Acme Corp",
                            "invoice_number": "INV-001",
                            "total_amount": 1500,
                        }
                    },
                }
            ],
        )
        """
        _response = self._raw_client.create(evaluation_set_id, items=items, request_options=request_options)
        return _response.data

    def retrieve(
        self, evaluation_set_id: str, item_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EvaluationSetItem:
        """
        Get details of an evaluation set item.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        item_id : str
            The ID of the evaluation set item.

            Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItem
            Evaluation set item details

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.evaluation_set_items.retrieve(
            evaluation_set_id="evaluation_set_id_here",
            item_id="evaluation_set_item_id_here",
        )
        """
        _response = self._raw_client.retrieve(evaluation_set_id, item_id, request_options=request_options)
        return _response.data

    def update(
        self,
        evaluation_set_id: str,
        item_id: str,
        *,
        expected_output: ProvidedProcessorOutputParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EvaluationSetItem:
        """
        If you need to change the expected output for a given evaluation set item, you can use this endpoint to update the item. This can be useful if you need to correct an error in the expected output or if the output of the extractor, classifier, or splitter has changed.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        item_id : str
            The ID of the evaluation set item.

            Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`

        expected_output : ProvidedProcessorOutputParams
            The expected output of the extractor, classifier, or splitter when run against the file. This must conform to the output schema of the entity associated with the evaluation set.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItem
            Successfully updated evaluation set item

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.evaluation_set_items.update(
            evaluation_set_id="evaluation_set_id_here",
            item_id="evaluation_set_item_id_here",
            expected_output={
                "value": {
                    "vendor_name": "Acme Corp",
                    "invoice_number": "INV-001",
                    "total_amount": 1750,
                }
            },
        )
        """
        _response = self._raw_client.update(
            evaluation_set_id, item_id, expected_output=expected_output, request_options=request_options
        )
        return _response.data

    def delete(
        self, evaluation_set_id: str, item_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EvaluationSetItemsDeleteResponse:
        """
        Delete an evaluation set item.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        item_id : str
            The ID of the evaluation set item.

            Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItemsDeleteResponse
            Evaluation set item deleted successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.evaluation_set_items.delete(
            evaluation_set_id="evaluation_set_id_here",
            item_id="evaluation_set_item_id_here",
        )
        """
        _response = self._raw_client.delete(evaluation_set_id, item_id, request_options=request_options)
        return _response.data


class AsyncEvaluationSetItemsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEvaluationSetItemsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEvaluationSetItemsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEvaluationSetItemsClient
        """
        return self._raw_client

    async def list(
        self,
        evaluation_set_id: str,
        *,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EvaluationSetItemsListResponse:
        """
        List items in a specific evaluation set.

        Returns a summary of each evaluation set item. Use the [Get Evaluation Set Item](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/evaluation/get-evaluation-set-item) endpoint to get the full details of an evaluation set item.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItemsListResponse
            List of evaluation set items

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.evaluation_set_items.list(
                evaluation_set_id="evaluation_set_id_here",
                next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            evaluation_set_id,
            sort_by=sort_by,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        evaluation_set_id: str,
        *,
        items: typing.Sequence[EvaluationSetItemsCreateRequestItemsItemParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EvaluationSetItemsCreateResponse:
        """
        Evaluation set items are the individual files and expected outputs that are used to evaluate the performance of a given extractor, classifier, or splitter in Extend. This endpoint will create new evaluation set items in Extend, which will be used during an evaluation run.

        **Limit:** You can create up to 100 items at a time.

        Learn more about how to create evaluation set items in the [Evaluation Sets](https://docs.extend.ai/2026-02-09/product/evaluation/overview) product page.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        items : typing.Sequence[EvaluationSetItemsCreateRequestItemsItemParams]
            An array of objects representing the evaluation set items to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItemsCreateResponse
            Evaluation set items created successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.evaluation_set_items.create(
                evaluation_set_id="evaluation_set_id_here",
                items=[
                    {
                        "file_id": "file_xK9mLPqRtN3vS8wF5hB2cQ",
                        "expected_output": {
                            "value": {
                                "vendor_name": "Acme Corp",
                                "invoice_number": "INV-001",
                                "total_amount": 1500,
                            }
                        },
                    }
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(evaluation_set_id, items=items, request_options=request_options)
        return _response.data

    async def retrieve(
        self, evaluation_set_id: str, item_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EvaluationSetItem:
        """
        Get details of an evaluation set item.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        item_id : str
            The ID of the evaluation set item.

            Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItem
            Evaluation set item details

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.evaluation_set_items.retrieve(
                evaluation_set_id="evaluation_set_id_here",
                item_id="evaluation_set_item_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(evaluation_set_id, item_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        evaluation_set_id: str,
        item_id: str,
        *,
        expected_output: ProvidedProcessorOutputParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EvaluationSetItem:
        """
        If you need to change the expected output for a given evaluation set item, you can use this endpoint to update the item. This can be useful if you need to correct an error in the expected output or if the output of the extractor, classifier, or splitter has changed.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        item_id : str
            The ID of the evaluation set item.

            Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`

        expected_output : ProvidedProcessorOutputParams
            The expected output of the extractor, classifier, or splitter when run against the file. This must conform to the output schema of the entity associated with the evaluation set.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItem
            Successfully updated evaluation set item

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.evaluation_set_items.update(
                evaluation_set_id="evaluation_set_id_here",
                item_id="evaluation_set_item_id_here",
                expected_output={
                    "value": {
                        "vendor_name": "Acme Corp",
                        "invoice_number": "INV-001",
                        "total_amount": 1750,
                    }
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            evaluation_set_id, item_id, expected_output=expected_output, request_options=request_options
        )
        return _response.data

    async def delete(
        self, evaluation_set_id: str, item_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EvaluationSetItemsDeleteResponse:
        """
        Delete an evaluation set item.

        Parameters
        ----------
        evaluation_set_id : str
            The ID of the evaluation set.

            Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`

        item_id : str
            The ID of the evaluation set item.

            Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EvaluationSetItemsDeleteResponse
            Evaluation set item deleted successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.evaluation_set_items.delete(
                evaluation_set_id="evaluation_set_id_here",
                item_id="evaluation_set_item_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(evaluation_set_id, item_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.split_advanced_options_split_method import SplitAdvancedOptionsSplitMethod
from .page_ranges import PageRangesParams


class SplitAdvancedOptionsParams(typing_extensions.TypedDict):
    split_identifier_rules: typing_extensions.NotRequired[
        typing_extensions.Annotated[str, FieldMetadata(alias="splitIdentifierRules")]
    ]
    """
    Custom rules for identifying split points.
    
    **Deprecated:** For `splitting_light` >= 1.3.0 and `splitting_performance` >= 1.5.0, use the `identifierKey` field on each classification in `splitClassifications` instead. This provides per-type identifier extraction rules rather than a single global rule. On those versions, this field is accepted but ignored if provided.
    """

    split_method: typing_extensions.NotRequired[
        typing_extensions.Annotated[SplitAdvancedOptionsSplitMethod, FieldMetadata(alias="splitMethod")]
    ]
    """
    The method to use for splitting documents. `high_precision` is more accurate but slower, while `basic_precision` is faster but less precise.
    
    **Deprecated:** For `splitting_light` >= 1.3.0 and `splitting_performance` >= 1.5.0, this field has no impact and is ignored if provided. It is still accepted for compatibility with older integrations.
    """

    split_excel_documents_by_sheet_enabled: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="splitExcelDocumentsBySheetEnabled")]
    ]
    """
    For Excel documents, split by worksheet.
    """

    page_ranges: typing_extensions.NotRequired[
        typing_extensions.Annotated[PageRangesParams, FieldMetadata(alias="pageRanges")]
    ]
    page_overlap_enabled: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="pageOverlapEnabled")]
    ]
    """
    When enabled, the splitter will allow for page overlap when splitting a document such that a page can occur in multiple adjacent splits when context from the previous split is on the page and context from the next split is on the page.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...core.serialization import FieldMetadata


class ProcessorRunDeleteResponseParams(typing_extensions.TypedDict):
    success: bool
    document_processor_run_id: typing_extensions.Annotated[str, FieldMetadata(alias="documentProcessorRunId")]
    """
    The ID of the deleted processor run
    """

    message: str
    """
    Confirmation message
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .created_at import CreatedAt
from .file_metadata import FileMetadata
from .file_type import FileType
from .updated_at import UpdatedAt


class FileSummary(UncheckedBaseModel):
    object: typing.Literal["file"] = pydantic.Field(default="file")
    """
    The type of object. Will always be `"file"`.
    """

    id: str = pydantic.Field()
    """
    ID for the file.
    
    Example: `"file_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    name: str = pydantic.Field()
    """
    The name of the file
    
    Example: `"Invoices.pdf"`
    """

    type: typing.Optional[FileType] = pydantic.Field(default=None)
    """
    The type of the file.
    
    **Availability:** Present when the file type could be determined.
    """

    parent_file_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="parentFileId")] = (
        pydantic.Field(alias="parentFileId", default=None)
    )
    """
    ID of the parent file.
    
    **Availability:** Present for files created via a Splitter in a workflow.
    """

    metadata: FileMetadata
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")] = pydantic.Field(
        alias="updatedAt"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .file_summary import FileSummary


class EvaluationSetItemSummary(UncheckedBaseModel):
    object: typing.Literal["evaluation_set_item"] = pydantic.Field(default="evaluation_set_item")
    """
    The type of object. In this case, it will always be `"evaluation_set_item"`.
    """

    id: str = pydantic.Field()
    """
    The ID of the evaluation set item.
    
    Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`
    """

    file: FileSummary = pydantic.Field()
    """
    A summary of the file associated with the evaluation set item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.batch_run_status import BatchRunStatus
from ..types.created_at import CreatedAt
from ..types.updated_at import UpdatedAt
from .batch_run_entity import BatchRunEntityParams
from .batch_run_entity_version import BatchRunEntityVersionParams
from .batch_run_metrics import BatchRunMetricsParams


class BatchRunParams(typing_extensions.TypedDict):
    object: typing.Literal["batch_run"]
    """
    The type of object. In this case, it will always be `"batch_run"`.
    """

    id: str
    """
    The unique identifier for this batch run.
    
    Example: `"batch_run_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    entity: typing.Optional[BatchRunEntityParams]
    """
    The extractor, classifier, or splitter that was run.
    
    **Availability:** Present when an entity was provided when creating the batch run.
    """

    entity_version: typing_extensions.Annotated[
        typing.Optional[BatchRunEntityVersionParams], FieldMetadata(alias="entityVersion")
    ]
    """
    The version of the extractor, classifier, or splitter that was run.
    
    **Availability:** Present when an entity was provided when creating the batch run.
    """

    status: BatchRunStatus
    metrics: BatchRunMetricsParams
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class WorkflowReference(UncheckedBaseModel):
    """
    The workflow to run.
    """

    id: str = pydantic.Field()
    """
    The ID of the workflow.
    
    Example: `"workflow_BMdfq_yWM3sT-ZzvCnA3f"`
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional version of the workflow that files will be run through. This number can be found when viewing the workflow on the Extend platform. When a version number is not supplied, the most recent published version of the workflow will be used. If no published versions exist, the draft version will be used. To run the `"draft"` version of a workflow, use `"draft"` as the version.
    
    Examples:
    - `"3"` - Run version 3 of the workflow
    - `"draft"` - Run the draft version of the workflow
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError as core_api_error_ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.payment_required_error import PaymentRequiredError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.api_error import ApiError as types_api_error_ApiError
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_dir import SortDir
from ..types.webhook_subscription import WebhookSubscription
from ..types.webhook_subscription_event_type import WebhookSubscriptionEventType
from ..types.webhook_subscription_resource_type import WebhookSubscriptionResourceType
from .types.webhook_subscriptions_delete_response import WebhookSubscriptionsDeleteResponse
from .types.webhook_subscriptions_list_response import WebhookSubscriptionsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawWebhookSubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        webhook_endpoint_id: typing.Optional[str] = None,
        resource_id: typing.Optional[str] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WebhookSubscriptionsListResponse]:
        """
        List webhook subscriptions. You can filter by `webhookEndpointId` to see all subscriptions for a given endpoint, or by `resourceId` to see all subscriptions for a given resource.

        Parameters
        ----------
        webhook_endpoint_id : typing.Optional[str]
            Filter subscriptions by the webhook endpoint they belong to.

        resource_id : typing.Optional[str]
            Filter subscriptions by the resource they are scoped to.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookSubscriptionsListResponse]
            List of webhook subscriptions
        """
        _response = self._client_wrapper.httpx_client.request(
            "webhook_subscriptions",
            method="GET",
            params={
                "webhookEndpointId": webhook_endpoint_id,
                "resourceId": resource_id,
                "sortDir": sort_dir,
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscriptionsListResponse,
                    construct_type(
                        type_=WebhookSubscriptionsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def create(
        self,
        *,
        webhook_endpoint_id: str,
        resource_type: WebhookSubscriptionResourceType,
        resource_id: str,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WebhookSubscription]:
        """
        Create a resource-scoped webhook subscription on an existing webhook endpoint.

        Subscriptions let you receive events for a specific resource (e.g., a single extractor or workflow) rather than all resources of that type. The `enabledEvents` must be valid for the given `resourceType` and the endpoint's `apiVersion`.

        If a subscription already exists for the same endpoint and resource, it will be updated with the new `enabledEvents` instead of creating a duplicate.

        Parameters
        ----------
        webhook_endpoint_id : str
            The ID of the webhook endpoint to attach this subscription to.

        resource_type : WebhookSubscriptionResourceType

        resource_id : str
            The ID of the resource to scope this subscription to.

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the given `resourceType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookSubscription]
            Successfully created webhook subscription
        """
        _response = self._client_wrapper.httpx_client.request(
            "webhook_subscriptions",
            method="POST",
            json={
                "webhookEndpointId": webhook_endpoint_id,
                "resourceType": resource_type,
                "resourceId": resource_id,
                "enabledEvents": enabled_events,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscription,
                    construct_type(
                        type_=WebhookSubscription,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[WebhookSubscription]:
        """
        Retrieve a webhook subscription by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookSubscription]
            Retrieved webhook subscription
        """
        _response = self._client_wrapper.httpx_client.request(
            f"webhook_subscriptions/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscription,
                    construct_type(
                        type_=WebhookSubscription,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def update(
        self,
        id: str,
        *,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WebhookSubscription]:
        """
        Update the enabled events on a webhook subscription.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to update.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the subscription's resource type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookSubscription]
            Successfully updated webhook subscription
        """
        _response = self._client_wrapper.httpx_client.request(
            f"webhook_subscriptions/{jsonable_encoder(id)}",
            method="POST",
            json={
                "enabledEvents": enabled_events,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscription,
                    construct_type(
                        type_=WebhookSubscription,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[WebhookSubscriptionsDeleteResponse]:
        """
        Delete a webhook subscription. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to delete.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookSubscriptionsDeleteResponse]
            Successfully deleted webhook subscription
        """
        _response = self._client_wrapper.httpx_client.request(
            f"webhook_subscriptions/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscriptionsDeleteResponse,
                    construct_type(
                        type_=WebhookSubscriptionsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )


class AsyncRawWebhookSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        webhook_endpoint_id: typing.Optional[str] = None,
        resource_id: typing.Optional[str] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WebhookSubscriptionsListResponse]:
        """
        List webhook subscriptions. You can filter by `webhookEndpointId` to see all subscriptions for a given endpoint, or by `resourceId` to see all subscriptions for a given resource.

        Parameters
        ----------
        webhook_endpoint_id : typing.Optional[str]
            Filter subscriptions by the webhook endpoint they belong to.

        resource_id : typing.Optional[str]
            Filter subscriptions by the resource they are scoped to.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookSubscriptionsListResponse]
            List of webhook subscriptions
        """
        _response = await self._client_wrapper.httpx_client.request(
            "webhook_subscriptions",
            method="GET",
            params={
                "webhookEndpointId": webhook_endpoint_id,
                "resourceId": resource_id,
                "sortDir": sort_dir,
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscriptionsListResponse,
                    construct_type(
                        type_=WebhookSubscriptionsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def create(
        self,
        *,
        webhook_endpoint_id: str,
        resource_type: WebhookSubscriptionResourceType,
        resource_id: str,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WebhookSubscription]:
        """
        Create a resource-scoped webhook subscription on an existing webhook endpoint.

        Subscriptions let you receive events for a specific resource (e.g., a single extractor or workflow) rather than all resources of that type. The `enabledEvents` must be valid for the given `resourceType` and the endpoint's `apiVersion`.

        If a subscription already exists for the same endpoint and resource, it will be updated with the new `enabledEvents` instead of creating a duplicate.

        Parameters
        ----------
        webhook_endpoint_id : str
            The ID of the webhook endpoint to attach this subscription to.

        resource_type : WebhookSubscriptionResourceType

        resource_id : str
            The ID of the resource to scope this subscription to.

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the given `resourceType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookSubscription]
            Successfully created webhook subscription
        """
        _response = await self._client_wrapper.httpx_client.request(
            "webhook_subscriptions",
            method="POST",
            json={
                "webhookEndpointId": webhook_endpoint_id,
                "resourceType": resource_type,
                "resourceId": resource_id,
                "enabledEvents": enabled_events,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscription,
                    construct_type(
                        type_=WebhookSubscription,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[WebhookSubscription]:
        """
        Retrieve a webhook subscription by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookSubscription]
            Retrieved webhook subscription
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"webhook_subscriptions/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscription,
                    construct_type(
                        type_=WebhookSubscription,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def update(
        self,
        id: str,
        *,
        enabled_events: typing.Sequence[WebhookSubscriptionEventType],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WebhookSubscription]:
        """
        Update the enabled events on a webhook subscription.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to update.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        enabled_events : typing.Sequence[WebhookSubscriptionEventType]
            The event types to subscribe to. Must be valid for the subscription's resource type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookSubscription]
            Successfully updated webhook subscription
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"webhook_subscriptions/{jsonable_encoder(id)}",
            method="POST",
            json={
                "enabledEvents": enabled_events,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscription,
                    construct_type(
                        type_=WebhookSubscription,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[WebhookSubscriptionsDeleteResponse]:
        """
        Delete a webhook subscription. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook subscription to delete.

            Example: `"whes_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookSubscriptionsDeleteResponse]
            Successfully deleted webhook subscription
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"webhook_subscriptions/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookSubscriptionsDeleteResponse,
                    construct_type(
                        type_=WebhookSubscriptionsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.legacy_splitter_config_base_processor import LegacySplitterConfigBaseProcessor
from .legacy_classification import LegacyClassificationParams
from .legacy_splitter_advanced_options import LegacySplitterAdvancedOptionsParams
from .parse_config import ParseConfigParams


class LegacySplitterConfigParams(typing_extensions.TypedDict):
    base_processor: typing_extensions.NotRequired[
        typing_extensions.Annotated[LegacySplitterConfigBaseProcessor, FieldMetadata(alias="baseProcessor")]
    ]
    """
    The base processor to use. For splitters, this can currently only be `"splitting_performance"` or `"splitting_light"`. See [Splitting Changelog](https://docs.extend.ai/2026-02-09/changelog/splitting/splitting-performance) for more details.
    """

    base_version: typing_extensions.NotRequired[typing_extensions.Annotated[str, FieldMetadata(alias="baseVersion")]]
    """
    The version of the `"splitting_performance"` or `"splitting_light"` processor to use. If this is provided, the `baseProcessor` must also be provided. See [Splitting Changelog](https://docs.extend.ai/2026-02-09/changelog/splitting/splitting-performance) for more details.
    """

    split_classifications: typing_extensions.Annotated[
        typing.Sequence[LegacyClassificationParams], FieldMetadata(alias="splitClassifications")
    ]
    """
    Array of classifications that define the possible types of document sections.
    """

    split_rules: typing_extensions.NotRequired[typing_extensions.Annotated[str, FieldMetadata(alias="splitRules")]]
    """
    Custom rules to guide the document splitting process in natural language.
    """

    advanced_options: typing_extensions.NotRequired[
        typing_extensions.Annotated[LegacySplitterAdvancedOptionsParams, FieldMetadata(alias="advancedOptions")]
    ]
    """
    Advanced configuration options.
    """

    parser: typing_extensions.NotRequired[ParseConfigParams]
    """
    Configuration options for the parsing process.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.json_object import JsonObject
from .extract_output_metadata import ExtractOutputMetadataParams


class ExtractOutputJsonParams(typing_extensions.TypedDict):
    value: JsonObject
    """
    The extracted values conforming to the schema defined in the processor config
    """

    metadata: ExtractOutputMetadataParams



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class EditRunMetricsParams(typing_extensions.TypedDict):
    """
    Metrics about the editing process.

    **Availability:** Present when `status` is `"PROCESSED"`.
    """

    processing_time_ms: typing_extensions.Annotated[float, FieldMetadata(alias="processingTimeMs")]
    """
    Total processing time in milliseconds.
    """

    page_count: typing_extensions.Annotated[int, FieldMetadata(alias="pageCount")]
    """
    The number of pages in the document.
    """

    field_count: typing_extensions.Annotated[int, FieldMetadata(alias="fieldCount")]
    """
    The total number of fields in the schema.
    """

    fields_detected_count: typing_extensions.Annotated[int, FieldMetadata(alias="fieldsDetectedCount")]
    """
    The number of fields that were automatically detected.
    """

    fields_annotated_count: typing_extensions.Annotated[int, FieldMetadata(alias="fieldsAnnotatedCount")]
    """
    The number of fields that were annotated with bounding boxes.
    """

    field_detection_time_ms: typing_extensions.Annotated[float, FieldMetadata(alias="fieldDetectionTimeMs")]
    """
    The time taken to detect fields in the document, in milliseconds.
    """

    field_annotation_time_ms: typing_extensions.Annotated[float, FieldMetadata(alias="fieldAnnotationTimeMs")]
    """
    The time taken to annotate field positions in the document, in milliseconds.
    """

    field_filling_time_ms: typing_extensions.Annotated[float, FieldMetadata(alias="fieldFillingTimeMs")]
    """
    The time taken to fill the fields in the document, in milliseconds.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.created_at import CreatedAt
from ..types.file_type import FileType
from ..types.updated_at import UpdatedAt
from .file_metadata import FileMetadataParams


class FileSummaryParams(typing_extensions.TypedDict):
    object: typing.Literal["file"]
    """
    The type of object. Will always be `"file"`.
    """

    id: str
    """
    ID for the file.
    
    Example: `"file_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    name: str
    """
    The name of the file
    
    Example: `"Invoices.pdf"`
    """

    type: typing.Optional[FileType]
    """
    The type of the file.
    
    **Availability:** Present when the file type could be determined.
    """

    parent_file_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="parentFileId")]
    """
    ID of the parent file.
    
    **Availability:** Present for files created via a Splitter in a workflow.
    """

    metadata: FileMetadataParams
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]



"""
Extended ExtractRuns client with polling utilities.

Example:
    from extend_ai import Extend

    client = Extend(token="...")

    # Create and poll until completion
    result = client.extract_runs.create_and_poll(
        file={"id": "file_xxx"},
        extractor={"id": "extractor_abc123"},
    )

    if result.status == "PROCESSED":
        print(result.output)
"""

from typing import Any, Dict, Optional

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...extract_runs.client import AsyncExtractRunsClient as GeneratedAsyncExtractRunsClient
from ...extract_runs.client import ExtractRunsClient as GeneratedExtractRunsClient
from ...extract_runs.requests.extract_runs_create_request_extractor import ExtractRunsCreateRequestExtractorParams
from ...extract_runs.requests.extract_runs_create_request_file import ExtractRunsCreateRequestFileParams
from ...requests.extract_config_json import ExtractConfigJsonParams
from ...types.extract_run import ExtractRun
from ...types.run_metadata import RunMetadata
from ...types.run_priority import RunPriority
from ..polling import PollingOptions, poll_until_done, poll_until_done_async

# Re-export for convenience
from ..polling import PollingTimeoutError

__all__ = ["ExtractRunsClient", "AsyncExtractRunsClient", "PollingTimeoutError"]


def _is_terminal_status(status: str) -> bool:
    """
    Check if a ProcessorRunStatus is terminal (no longer processing).
    We check for non-terminal states rather than terminal states so that
    if new terminal states are added, polling will still complete.
    """
    return status not in ("PROCESSING", "PENDING", "CANCELLING")


class ExtractRunsClient(GeneratedExtractRunsClient):
    """
    Extended ExtractRuns client with create_and_poll method.

    Inherits all methods from ExtractRunsClient and adds create_and_poll
    for convenient polling until completion.
    """

    def __init__(self, *, client_wrapper: SyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)

    def create_and_poll(
        self,
        *,
        file: ExtractRunsCreateRequestFileParams,
        extractor: Optional[ExtractRunsCreateRequestExtractorParams] = None,
        config: Optional[ExtractConfigJsonParams] = None,
        priority: Optional[RunPriority] = None,
        metadata: Optional[RunMetadata] = None,
        polling_options: Optional[PollingOptions] = None,
    ) -> ExtractRun:
        """
        Creates an extract run and polls until it reaches a terminal state.

        This is a convenience method that combines create() and polling via
        retrieve() with exponential backoff and jitter.

        Terminal states: PROCESSED, FAILED, CANCELLED

        Args:
            file: The file to be extracted from.
            extractor: Reference to an existing extractor.
            config: Inline extract configuration.
            priority: Priority of the run.
            metadata: Additional metadata for the run.
            polling_options: Options for polling behavior.

        Returns:
            The final extract run when processing is complete.

        Raises:
            PollingTimeoutError: If the run doesn't complete within max_wait_ms.

        Example:
            result = client.extract_runs.create_and_poll(
                file={"id": "file_xxx"},
                extractor={"id": "extractor_abc123"}
            )

            if result.status == "PROCESSED":
                print(result.output)
        """
        # Build kwargs, only including non-None values to avoid passing null
        kwargs: Dict[str, Any] = {"file": file}
        if extractor is not None:
            kwargs["extractor"] = extractor
        if config is not None:
            kwargs["config"] = config
        if priority is not None:
            kwargs["priority"] = priority
        if metadata is not None:
            kwargs["metadata"] = metadata

        # Create the extract run
        create_response = self.create(**kwargs)
        run_id = create_response.id

        # Poll until terminal state
        return poll_until_done(
            retrieve=lambda: self.retrieve(run_id),
            is_terminal=lambda response: _is_terminal_status(response.status),
            options=polling_options,
        )


class AsyncExtractRunsClient(GeneratedAsyncExtractRunsClient):
    """
    Extended AsyncExtractRuns client with create_and_poll method.
    """

    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)

    async def create_and_poll(
        self,
        *,
        file: ExtractRunsCreateRequestFileParams,
        extractor: Optional[ExtractRunsCreateRequestExtractorParams] = None,
        config: Optional[ExtractConfigJsonParams] = None,
        priority: Optional[RunPriority] = None,
        metadata: Optional[RunMetadata] = None,
        polling_options: Optional[PollingOptions] = None,
    ) -> ExtractRun:
        """
        Creates an extract run and polls until it reaches a terminal state (async version).
        """
        # Build kwargs, only including non-None values to avoid passing null
        kwargs: Dict[str, Any] = {"file": file}
        if extractor is not None:
            kwargs["extractor"] = extractor
        if config is not None:
            kwargs["config"] = config
        if priority is not None:
            kwargs["priority"] = priority
        if metadata is not None:
            kwargs["metadata"] = metadata

        # Create the extract run
        create_response = await self.create(**kwargs)
        run_id = create_response.id

        # Poll until terminal state
        return await poll_until_done_async(
            retrieve=lambda: self.retrieve(run_id),
            is_terminal=lambda response: _is_terminal_status(response.status),
            options=polling_options,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class ApiError(UncheckedBaseModel):
    """
    Standard error response format for all Extend API errors.

    See the [Error Codes documentation](https://docs.extend.ai/2026-02-09/developers/error-codes) for error handling recommendations.
    """

    code: str = pydantic.Field()
    """
    Error code for programmatic handling.
    """

    message: str = pydantic.Field()
    """
    Human-readable error message suitable for displaying to developers. 
    Do not rely on the exact text of this message as it may change.
    """

    retryable: bool = pydantic.Field()
    """
    Whether the request can be retried. When true, retry with exponential backoff.
    When false, fix the underlying issue before retrying.
    """

    request_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="requestId")] = pydantic.Field(
        alias="requestId", default=None
    )
    """
    Unique request identifier for support purposes. Always include this 
    when contacting Extend support about an error.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ChunkType(enum.StrEnum):
    """
    The type of chunk.
    """

    PAGE = "page"
    DOCUMENT = "document"
    SECTION = "section"
    _UNKNOWN = "__CHUNKTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ChunkType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        page: typing.Callable[[], T_Result],
        document: typing.Callable[[], T_Result],
        section: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ChunkType.PAGE:
            return page()
        if self is ChunkType.DOCUMENT:
            return document()
        if self is ChunkType.SECTION:
            return section()
        return _unknown_member(self._value_)
